/**
 * @class OSCBundleHandler
 *
 * Dispatch the contents of a bundle.
 *
 * Daniel Holth
 * 12 Nov. 2002
 */

#ifndef OSCBUNDLE_HANDLER_H
#define OSCBUNDLE_HANDLER_H

#include "OSCPacker.h"
#include "OSCUnpacker.h"
#include "OSCNamespace.h"
#include "OSCCallable.h"
#include "OSCProcessor.h"

class OSCBundleHandler : public OSCCallable
{
  private:
    OSCProcessor* myProcessor;
  protected:
    OSCProcessor* getProcessor()        { return this->myProcessor; }
  public:
    OSCBundleHandler()          { }
    virtual ~OSCBundleHandler() { }

    /// Set the processor
    /** Assign this to the normal processor that dispatches incoming messages. */
    void setProcessor(OSCProcessor *p)  { this->myProcessor = p; }
    
    void call(const std::string& data, Transmit *const replier) 
    {
      bool success;
      std::string queryAddress;
      std::string itemAddress;
      std::string itemData;
      OSCProcessor *processor;
      OSCUnpacker   u;
      OSCUnpacker   item;

      u.setData(data);

      // DEBUG
      u.unpackString(&queryAddress);               // Probably #bundle
      std::cout << "Bundle: " << queryAddress << std::endl;
      // END DEBUG

      u.skipHyper();                  // Ignore 8-byte time tag
      
      processor = this->getProcessor();

      success = true;

      while (success)
      {
        success = u.unpackBlob(&itemData);
        if(success)
        {
          // DEBUG
          item.setData(itemData);
          item.unpackString(&itemAddress);
          std::cout << "Bundle item: " << itemAddress << std::endl;
          // END DEBUG
          
          processor->process(itemData, replier);
        }
      }
    };
};

#endif
