/* Open SoundControl kit in C++                                              */
/* Copyright (C) 2002 Daniel Holth, Clinton McChesney                        */
/*                                                                           */
/* This library is free software; you can redistribute it and/or             */
/* modify it under the terms of the GNU Lesser General Public                */
/* License as published by the Free Software Foundation; either              */
/* version 2.1 of the License, or (at your option) any later version.        */
/*                                                                           */
/* This library is distributed in the hope that it will be useful,           */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of            */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU         */
/* Lesser General Public License for more details.                           */
/*                                                                           */
/* You should have received a copy of the GNU Lesser General Public          */
/* License along with this library; if not, write to the Free Software       */
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
/*                                                                           */
/* For questions regarding this program contact                              */
/* Daniel Holth <dholth@stetson.edu> or visit                                */
/* http://www.stetson.edu/~ProctoLogic/                                      */

/**
 * @file MasterBroadcaster.cc
 *
 * @author Daniel Holth
 * @author Clinton McChesney
 */

#include "MasterBroadcaster.h"

MasterBroadcaster::MasterBroadcaster()
{
  sem_t*  semaphore;

  semaphore = this->getSemaphore();
  sem_init(semaphore,0,0);

  this->myBroadcasters = new BroadcasterNamespace();
}


MasterBroadcaster::~MasterBroadcaster()
{
  sem_t*  semaphore;

  semaphore = this->getSemaphore();
  sem_destroy(semaphore);

  delete(this->myBroadcasters);
}


void* run_broadcaster(void* arg)
{
  MasterBroadcaster* mb;

  mb = (MasterBroadcaster*)arg;
  
  mb->run();

  return NULL;
}


void MasterBroadcaster::startThread()
{
  pthread_t* thread;

  thread = this->getThread();
  
  //pthread_create(thread,PTHREAD_CREATE_DETACHED,run_broadcaster,(void*)this);
  pthread_create(thread, NULL, run_broadcaster, (void*)this);
  pthread_detach(*thread);
}


void MasterBroadcaster::run()
{
  sem_t*                                         queuedTotal;
  BroadcasterNamespace*                          bn;
  BroadcasterNamespace::BroadcasterMap*          broadcasters;
  BroadcasterNamespace::BroadcasterMap::iterator iterator;
  BroadcasterNamespace::BroadcasterMap::iterator end;
  bool sent;
 
  queuedTotal  = this->getSemaphore();
  bn           = this->getBroadcasters();
  broadcasters = bn->getBroadcasters();
  
  end      = broadcasters->end();

  /*
   * one: wait on the semaphore (wet)
   * two: iterate through broadcasters until a message can be sent(lather)
   * three: send the message(rinse)
   * four: repeat(repeat)
   */
 
  while(true)
  {
    iterator = broadcasters->begin();
    // Iterating through the broadcasters:
    sent = true;
    while (iterator != end)
    {
      if(sent)
      {
        sem_wait(queuedTotal);
        sent = false;
      }
      sent = ((QueuedBroadcaster*)iterator->second)->sendOne();
      iterator++;
    }

    if(!sent)
    {
      sem_post(queuedTotal);
    }
  }
}
